package # hide from pause?
 My::Test::Project;

=head1 NAME

My::Test::Project - can do stuff

=head1 VERSION

42.00

=cut

use strict;
use warnings;
use Applify;
use App::Mypp;

our $VERSION = eval '42.00';

=head1 AUTHOR

See L<App::Mypp>.

=cut

1;
